import {
  Button,
  LinearProgress,
  Grid,
  Typography,
  Tooltip
} from "@mui/material";
import { styled } from "@mui/material/styles";
import React, { useState, useEffect, useRef, useContext } from "react";
import { useStartIOC, useStopIOC } from "../../../api/SwaggerApi";
import { SimpleModal } from "../../common/SimpleModal/SimpleModal";
import { ConfirmationDialog } from "../../dialog/ConfirmationDialog";
import { notificationContext } from "../../common/notification/Notifications";
import Alert from "@mui/material/Alert";
import { initRequestParams } from "../../common/Helper";
import AccessControl from "../../auth/AccessControl";
import { useNavigate } from "react-router-dom";

const StartButton = styled(Button)(({ theme }) => ({
  backgroundColor: theme.palette.essGrass.main,
  color: theme.palette.essWhite.main,
  "&:disabled": {
    backgroundColor: theme.palette.grey[300],
    color: theme.palette.grey[500]
  },
  "&:hover": {
    backgroundColor: theme.palette.essGrass.main,
    color: theme.palette.essWhite.main
  }
}));
const StopButton = styled(Button)(({ theme }) => ({
  backgroundColor: theme.palette.essNavy.main,
  color: theme.palette.essWhite.main,
  "&:disabled": {
    backgroundColor: theme.palette.grey[300],
    color: theme.palette.grey[500]
  },
  "&:hover": {
    backgroundColor: theme.palette.essNavy.main,
    color: theme.palette.essWhite.main
  }
}));

export function IOCService({
  ioc,
  currentCommand,
  getOperations,
  buttonDisabled,
  setButtonDisabled,
  jobLazyParams
}) {
  const [error, setError] = useState();
  const navigate = useNavigate();

  function onError(message) {
    setError(message);
  }

  const [startJob, startIOC, resetStartJob] = useStartIOC(ioc.id, onError);
  const [stopJob, stopIOC, resetStopJob] = useStopIOC(ioc.id, onError);
  const [inProgress, setInProgress] = useState(false);
  const [adHocDialogOpen, setAdHocDialogOpen] = useState(false);
  const [adHocDialogTitle, setAdHocDiatlogTitle] = useState();
  const [adHocDialogDescription, setAdHocDialogDescription] = useState();
  const [command, setCommand] = useState(null);
  const callbackRef = useRef();
  const { watchCommand } = useContext(notificationContext);

  useEffect(() => {
    if (startJob && (!command || command.id !== startJob.id)) {
      watchCommand(startJob.id);
      navigate(`/jobs/${startJob.id}`);
      setCommand(startJob);
    } else if (stopJob && (!command || command.id !== stopJob.id)) {
      watchCommand(stopJob.id);
      navigate(`/jobs/${stopJob.id}`);
      setCommand(stopJob);
    } else if (currentCommand) {
      watchCommand(currentCommand.id);
      setCommand(currentCommand);
    }
  }, [startJob, stopJob, command, watchCommand, currentCommand, navigate]);

  function resetUI() {
    setCommand(null);
    resetStartJob();
    resetStopJob();
    setError(null);
  }

  const start = async () => {
    resetUI();
    setInProgress(true);
    setButtonDisabled(true);
    await startIOC();
    let requestParams = initRequestParams(jobLazyParams);
    requestParams.deployment_id = ioc.activeDeployment?.id;
    getOperations(requestParams);
  };

  const stop = async () => {
    resetUI();
    setInProgress(true);
    setButtonDisabled(true);
    await stopIOC();

    let requestParams = initRequestParams(jobLazyParams);

    requestParams.deployment_id = ioc.activeDeployment?.id;
    getOperations(requestParams);
  };

  const openStartModal = () => {
    setAdHocDiatlogTitle("Starting IOC");
    setAdHocDialogDescription(
      <>
        <Typography style={{ display: "inline-block" }}>
          Are you sure want to start &nbsp;
        </Typography>
        <Typography
          style={{ fontFamily: "monospace", display: "inline-block" }}
        >
          {" "}
          {ioc.namingName}?{" "}
        </Typography>
      </>
    );
    callbackRef.current = start;
    setAdHocDialogOpen(true);
  };

  const openStopModal = () => {
    setAdHocDiatlogTitle("Stopping IOC");
    setAdHocDialogDescription(
      <>
        <Typography style={{ display: "inline-block" }}>
          Are you sure want to stop &nbsp;
        </Typography>
        <Typography
          style={{ fontFamily: "monospace", display: "inline-block" }}
        >
          {" "}
          {ioc.namingName}?{" "}
        </Typography>
      </>
    );
    callbackRef.current = stop;
    setAdHocDialogOpen(true);
  };

  let disabledStartButtonTitle = "";
  if (buttonDisabled || ioc.operationInProgress) {
    disabledStartButtonTitle =
      "There is an ongoing operation, you can not Start the IOC right now";
  }

  let disabledStopButtonTitle = "";
  if (buttonDisabled || ioc.operationInProgress) {
    disabledStopButtonTitle =
      "There is an ongoing operation, you can not Stop the IOC right now";
  }

  return (
    <AccessControl
      allowedRoles={["DeploymentToolAdmin", "DeploymentToolIntegrator"]}
      renderNoAccess={() => "You are not authorized to control this IOC"}
    >
      <SimpleModal
        open={adHocDialogOpen}
        setOpen={setAdHocDialogOpen}
      >
        <ConfirmationDialog
          open={adHocDialogOpen}
          setOpen={setAdHocDialogOpen}
          title={adHocDialogTitle}
          description={adHocDialogDescription}
          callback={callbackRef.current}
        />
      </SimpleModal>

      <Grid
        container
        spacing={1}
      >
        <Grid item>
          <Tooltip title={disabledStopButtonTitle}>
            <StopButton
              onClick={openStopModal}
              disabled={buttonDisabled}
            >
              Stop
            </StopButton>
          </Tooltip>
        </Grid>
        <Grid item>
          <Tooltip title={disabledStartButtonTitle}>
            <StartButton
              onClick={openStartModal}
              disabled={buttonDisabled}
            >
              Start
            </StartButton>
          </Tooltip>
        </Grid>
        <Grid
          item
          xs={12}
          md={12}
        >
          {error ? (
            <Alert severity="error">{error}</Alert>
          ) : (
            inProgress && <LinearProgress color="primary" />
          )}
        </Grid>
      </Grid>
    </AccessControl>
  );
}
